/*****************************************************************************
 *
 * FILE:	calendar.c
 * DESCRIPTION:	Calendar CGI
 * DATE:	Sun, Apr  7 2002
 * UPDATE:	Sun, Aug 19 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2002-2007 졿Kouichi ABE (WALL), All rights reserved.
 * COMPILE:	gcc -export-dynamic calendar.c -o calendar.cgi
 *		-I/usr/local/include -L/usr/local/lib
 *		-lcockatrice -lcatoblepas -lwkf -lmd
 * $Id: calendar.c,v 1.3 2007/08/19 16:50:38 kouichi Exp $
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <cockatrice.h>
#include <catoblepas.h>

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
#define	LEAP(x)	((((x) % 4 == 0) && ((x) % 100 != 0)) || ((x) % 400 == 0))

/*****************************************************************************
 *
 *	Local functions definition
 *
 *****************************************************************************/
static void	makeCalendar(CGI *, HTML *, int, int);

/*****************************************************************************
 *
 *	Local variables declaration
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * FUNCTION NAME:	makeCalendar
 * DESCRIPTION:		make a calendar
 * ARGUMENTS:		CGI, HTML, year, month
 * RETURN VALUE:	none
 */
static void
makeCalendar(cgi, html, year, month)
	register CGI *	cgi;
	register HTML *	html;
	int		year;
	int		month;
{
#define	WDAYS	7
#if	0
  static String	wday[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
#else
  static char *	wday[] = {"", "", "", "", "", "", ""};
#endif
  static int	mday[] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
  static String	months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
			     "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

  int		firstDay = cgi->date->getDayOfTheWeek(year, month, 1);
  int		leap	 = LEAP(year);
  String	title;
  int		i;
  bool		lieu;
  int		d;

  mday[2] += leap;	/* revise */

  html->table.attr.border = 0;
  html->table.begin();
  asprintf(&title, "%s %d", months[month - 1], year);
  html->CAPTION(title);
  free(title);
  /* print a day of the week */
  html->tr.begin();
  for (i = 0; i < WDAYS; i++) {
    html->th.begin();
    html->th.attr.align = ALIGN_CENTER;
    html->th.attr.trans.width = 80;
    html->printf("%s", wday[i]);
    html->th.end();
  }
  html->tr.end();
  /* print a days of a month */
  html->tr.begin();
  html->td.attr.align = ALIGN_RIGHT;
  html->td.attr.valign = ALIGN_TOP;
  html->td.attr.trans.width = 80;
  html->td.attr.trans.height = 50;
  for (i = 0; i < firstDay; i++) {	/* padding */
    html->td.begin();
    html->print("&nbsp;");
    html->td.end();
  }
  for (lieu = false, d = 1; d <= mday[month]; d++, i++) {
    const char *	rokuyou;
    const char *	sekki;

    rokuyou = cgi->date->getRokuyou(year, month, d);
    sekki   = cgi->date->get24Sekki(year, month, d);
    if (i % WDAYS == 0) {
      html->tr.begin();
    }
    if (cgi->date->isFestival(year, month, d)) {	/* Festival ?*/
      if (cgi->date->getDayOfTheWeek(year, month, d) == 0) {	/* Sunday? */
	lieu = true;
      }
      html->td.begin();
      html->printf("$%d", d);
      html->printf("&nbsp;%s", rokuyou);
      html->printf("<br>%s", cgi->date->getFestivalName(year, month, d));
      if (sekki) {
	html->printf("<br>%s", sekki);
      }
      html->td.end();
    }
    else {	/* Weekday */
      html->td.begin();
      if (lieu) {	/* Festival in lieu */
	lieu = false;
	html->printf("+%d", d);
      }
      else {	/* Weekday */
	switch (i % WDAYS) {
	  case 0:	/* Sunday */
	    html->printf("*%d", d);
	    break;
	  case 6:	/* Saturday */
	    html->printf("#%d", d);
	    break;
	  default:
	    html->printf("%d", d);
	    break;
	}
      }
      html->printf("&nbsp;%s", rokuyou);
      if (sekki) {
	html->printf("<br>%s", sekki);
      }
      html->td.end();
    }
    if (i % WDAYS == 6) {
      html->tr.end();
    }
  }
  mday[2] -= leap;	/* end of revised */
  html->tr.end();
  html->table.end();
}


int
main(argc, argv)
	int	argc;
	char *	argv[];
{
  CGI *	cgi;	/* cgi object */
  HTML *	html;
  int	year;
  int	month;

  cgi = newCGI(CC_MODULE_DATE);
  if (cgi == NULL) {
    fprintf(stderr, "Error: %d\n", cgi_errno);
    return -1;
  }

  html = newHTML401(HTML4_Transitional);
  if (html) {
    if (cgi->param("year") != NULL) {
      year = atoi(cgi->param("year"));
    }
    else {
      year = cgi->date->year();
    }

    if (cgi->param("month") != NULL) {
      month = atoi(cgi->param("month"));
    }
    else {
      month = cgi->date->month();
    }

    cgi->header("text/html; charset=EUC-JP");

    html->html.begin();
      html->head.begin();
	html->meta.http_equiv("Content-Type", "text/html; charset=EUC-JP");
	html->title("Calendar");
      html->head.end();
      html->body.begin();
	html->H1("Calendar by Cockatrice");
	makeCalendar(cgi, html, year, month);
      html->body.end();
    html->html.end();

    html->done();
  }
  cgi->done();

  return 0;
}
